

#include "RotateControlBase.h"
#include "port/timer/wch//TimerField.h"

// Implementation *************************************************************************************************

static void doRotate(RotateControl *_controller, int N, float speedFactor)
{
    MotorControlBase *controller = (MotorControlBase*)&_controller->controller;
    LinRotAccelerator *acclerator = (LinRotAccelerator *)&_controller->accelerator;
    
    if(Controller_isRunning(controller))
    {
        // this->err(mcErr::alrdyMoving);
        if(!controller->errorCallback) return;
        controller->errorCallback(controller->leadMotor, err_running);
        return;
    }

    //Calculate Bresenham parameters ----------------------------------------------------------------
    sort_element(controller->motorList, controller->motorList + N, Stepper_cmpVmax);
    controller->leadMotor = controller->motorList[0];

    if (controller->leadMotor->vMax == 0)
        return;

    controller->leadMotor->currentSpeed = 0;

    controller->leadMotor->A = labs(controller->leadMotor->vMax);
    for (int i = 1; i < N; i++)
    {
        controller->motorList[i]->A = labs(controller->motorList[i]->vMax);
        controller->motorList[i]->B = 2 * controller->motorList[i]->A - controller->leadMotor->A;
    }
    uint32_t acceleration = (*(find_min_element(controller->motorList, controller->motorList + N, Stepper_cmpAcc)))->a; // use the lowest acceleration for the move

    // Start moving----------------------------------------------------------------------------------------------

    TimerField_begin(&controller->timerField);
    RotAccelerator_prepareRotation(acclerator, controller->leadMotor->current, controller->leadMotor->vMax, acceleration, controller->accUpdatePeriod, speedFactor);
    TimerField_accTimerStart(&controller->timerField);
}


void RotateControl_init(RotateControl *controller, const RotateControl_Init_TypeDef *config)
{
    Controller_init(&controller->controller, config);
    controller->controller.mode = MOTOR_NOTARGET;
}


// ISR -----------------------------------------------------------------------------------------------------------


void RotateControl_accTimerISR(RotateControl *_controller)
{
    MotorControlBase *controller = (MotorControlBase*)&_controller->controller;
    LinRotAccelerator *acceletator = (LinRotAccelerator*)&_controller->accelerator;
    
    if(controller->mode != MOTOR_NOTARGET) return;

    int32_t newSpeed = RotAccelerator_updateSpeed(acceletator, controller->leadMotor->current); // get new speed for the leading motor

    if((newSpeed == 0) || controller->timerField.lastPulse)
    {
        TimerField_end(&controller->timerField);
        controller->leadMotor->currentSpeed = 0;
        if(!controller->reachedTargetCallback) return;
        controller->reachedTargetCallback(controller->leadMotor->current);
        return;
    }

    if (controller->leadMotor->currentSpeed == newSpeed)
    {
        return; // nothing changed, just keep running
    }

    int dir = newSpeed >= 0 ? 1 : -1; // direction changed? -> toggle direction of all motors
    if (dir != controller->leadMotor->dir)
    {
#if(0)
        Stepper** motor = controller->motorList;
        while ((*motor) != NULL)
        {
            // (*motor++)->toggleDir();
        }

        delayMicroseconds(controller->pulseWidth); // dir signal need some lead time
#else
        controller->changeDir = 1;
#endif
    }

    TimerField_setStepFrequency(&controller->timerField, labs(newSpeed)); // speed changed, update timer
    if(controller->timerField.stepTimerRunning == 0){
        TimerField_stepTimerStart(&controller->timerField);
    }
    controller->leadMotor->currentSpeed = newSpeed;
}

// ROTATE Commands -------------------------------------------------------------------------------

void vRotateControl_rotateAsync(RotateControl *_controller, float speedFactor, uint8_t N, ...)
{
    MotorControlBase *controller = &_controller->controller;
    va_list mlist;

    va_start(mlist, N);
    vvController_attachStepper(controller, N, mlist);
    va_end(mlist);

    doRotate(_controller, N, speedFactor);
}


void RotateControl_rotateAsync(RotateControl *_controller, float speedFactor, uint8_t N, Stepper* *steppers)
{
    MotorControlBase *controller = &_controller->controller;

    Controller_attachStepper(controller, N, steppers);

    doRotate(_controller, N, speedFactor);
}


void RotateControl_overrideSpeed(RotateControl *controller, float factor)
{
    // isStopping = false;
    LinRotAccelerator *accelerator = &controller->accelerator;
    RotAccelerator_overrideSpeed(accelerator, factor);
}


void RotateControl_overrideAcceleration(RotateControl *controller, float factor)
{
    LinRotAccelerator *accelerator = &controller->accelerator;

    RotAccelerator_overrideAcceleration(accelerator, factor);
}


void RotateControl_stopAsync(RotateControl *_controller)
{
    // if (!isStopping)
    // {
        // isStopping = true;
    MotorControlBase *controller = &_controller->controller;
    LinRotAccelerator *accelerator = &_controller->accelerator;
    
    if(TimerField_accTimerIsRunning(&controller->timerField) == false)
        return;

    RotAccelerator_initiateStopping(accelerator, controller->leadMotor->current);

    TimerField_timerEndAfterPulse(&controller->timerField);
    // }
}

void RotateControl_stop(RotateControl *_controller)
{
    const MotorControlBase *controller = &_controller->controller;

    RotateControl_stopAsync(_controller);
    while (Controller_isRunning(controller))
    {
        delay(1);
    }
}

