
#include "TimerField.h"


static int instances = 0;

static TIM_Unit timer_mapping[] = TIMER_UNIT();

#define TIM_CLK_PSC      36000000
#define STEP_TIM_CLK     36000000
#define ACC_TIM_CLK      1000000

TIM_Unit TimerField_getTimer(void){
    // TODO
    return (instances >= (sizeof(timer_mapping) / sizeof(TIM_Unit))) ? timer_mapping[0] : timer_mapping[instances++];
}

void TimerField_init(TimerField *_timerfield, const TimerField_InitTypeDef *config){
    // TODO
    if(config){
        _timerfield->accTimer = config->accTimer;
        _timerfield->stepTimer = config->stepTimer;
        _timerfield->pulseTimer = config->pulseTimer;
    }else{
        _timerfield->stepTimer = TimerField_getTimer();
        _timerfield->accTimer = TimerField_getTimer();
        _timerfield->pulseTimer = TimerField_getTimer();
    }

    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure = { 0 };
    TIM_OCInitTypeDef TIM_OCInitStructure={ 0 };
    RCC_ClocksTypeDef RCC_Clocks = { 0 };
    
    uint32_t tim_clk = 144000000, priod = 144, presc = 1;
#if(ALLOCAT_PULSE_TIMER)
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
#endif
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
    
    RCC_GetClocksFreq(&RCC_Clocks);
    tim_clk = RCC_Clocks.PCLK1_Frequency;
    if(!(RCC->CFGR0 & RCC_PPRE1_DIV1)){
        tim_clk <<= 1;
    }

    tim_clk /= 4;
    presc = (tim_clk / STEP_TIM_CLK) - 1;
    priod = 27;   /// 默认间隔 3US

    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV4;
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInitStructure.TIM_Period = priod;
    TIM_TimeBaseInitStructure.TIM_Prescaler = 0;

#if(ALLOCAT_PULSE_TIMER)
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseInitStructure);
    TIM_ARRPreloadConfig(TIM2, DISABLE);
    TIM_Cmd(TIM2, DISABLE );
#endif
    /// 计数频率 9MHz, 用于输出 Step 脉冲
    TIM_TimeBaseInit(TIM4, &TIM_TimeBaseInitStructure);
    TIM_ARRPreloadConfig(TIM4, DISABLE);
    TIM_Cmd(TIM4, DISABLE );
    TIM_ClearFlag(TIM3, TIM_FLAG_Update);

    /// 用于设置加速间隔
    TIM_TimeBaseInitStructure.TIM_Prescaler = (tim_clk / ACC_TIM_CLK) - 1;
    TIM_TimeBaseInitStructure.TIM_Period = ACC_TIM_CLK / 200;   /// 默认间隔 200Hz
    TIM_TimeBaseInit(TIM3, &TIM_TimeBaseInitStructure);
    TIM_ARRPreloadConfig(TIM3, DISABLE);
    TIM_Cmd(TIM3, DISABLE );
    TIM_ClearFlag(TIM3, TIM_FLAG_Update);

#if(ALLOCAT_PULSE_TIMER == 0)
    TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
    TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
    TIM_OCInitStructure.TIM_Pulse = STEP_TIM_CLK / 500000;   /// 默认宽度 2US
    TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
    TIM_OC1Init( TIM4, &TIM_OCInitStructure );
    TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Disable);
#endif
    _timerfield->stepTimerRunning = 0;
    _timerfield->pulseTimerRunning = 0;
    _timerfield->accTimerRunning = 0;
    _timerfield->lastPulse = 0;
}

bool TimerField_begin(TimerField* timerfield){
    // TODO
#if(ALLOCAT_PULSE_TIMER == 0)
    TIM_ClearFlag(TIM4, TIM_FLAG_CC1);
    TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);
    TIM_ARRPreloadConfig(TIM4, DISABLE);
    TIM_ARRPreloadConfig(TIM3, DISABLE);
#endif
    TIM_SetCounter(TIM4, 0);
    TIM_SetCounter(TIM3, 0);
    timerfield->lastPulse = 0;
    return true;
}

void FUN_IN_RAM TimerField_end(TimerField* timerfield){
    // TODO
    TimerField_stepTimerStop(timerfield);
    TimerField_accTimerStop(timerfield);
    TimerField_pulseTimerStop(timerfield);
    timerfield->lastPulse = 0;
}

#define TIMERFIELD_START(tim)  do{ \
    TIM_ARRPreloadConfig(tim, ENABLE); \
    TIM_ClearFlag(tim, TIM_FLAG_Update); \
    TIM_ITConfig(tim, TIM_IT_Update, ENABLE); \
    TIM_Cmd(tim, ENABLE); \
}while(0)

#define TIMERFIELD_STOP(tim)  do{ \
    TIM_Cmd(tim, DISABLE); \
    TIM_ClearFlag(tim, TIM_FLAG_Update); \
    TIM_ITConfig(tim, TIM_IT_Update, DISABLE); \
}while(0)

void TimerField_stepTimerStart(TimerField* timerfield){
    // TODO
    TIMERFIELD_START(TIM4);
    timerfield->stepTimerRunning = 1;
}

void FUN_IN_RAM TimerField_stepTimerStop(TimerField* timerfield){
    // TODO
    TIMERFIELD_STOP(TIM4);
    timerfield->stepTimerRunning = 1;
}
bool TimerField_stepTimerIsAllocated(const TimerField* timerfield){
    // TODO
    (void)timerfield;
    return true;
}

bool TimerField_stepTimerIsRunning(const TimerField* timerfield){
    return timerfield->stepTimerRunning;
}

int32_t TimerField_getStepFrequency(const TimerField* timerfield){
    // TODO
    
    return STEP_TIM_CLK / (TIM4)->ATRLR;
}

void FUN_IN_RAM TimerField_setStepFrequency(TimerField* timerfield, uint32_t f){   // Hz
    // TODO
    uint32_t period = 0;
    uint32_t ccr = 0;
    ccr = TIM_GetCapture1(TIM4);
    if(f == 0){
        TimerField_stepTimerStop(timerfield);
        return;
    }else if(f < 138){
        f = 138;
        period = STEP_TIM_CLK / 138;
    }else if(f >= (STEP_TIM_CLK / (ccr + 9))){
        period = ccr + 9;
    }else{
        period = STEP_TIM_CLK / f;
    }
    // printf("%d, %ld\n", f, period);
    TIM_SetAutoreload(TIM4, period);
}

void TimerField_accTimerStart(TimerField* timerfield){
    // TODO
    TIMERFIELD_START(TIM3);
    timerfield->accTimerRunning = 1;
}

void FUN_IN_RAM TimerField_accTimerStop(TimerField* timerfield){
    // TODO
    TIMERFIELD_STOP(TIM3);
    timerfield->accTimerRunning = 0;
}

void FUN_IN_RAM TimerField_setAccUpdatePeriod(TimerField* timerfield, uint32_t _period){  // us
    // TODO
    uint32_t period = 0;
    if(_period == 0){
        TimerField_accTimerStop(timerfield);
        return;
    }else if(_period < 1){
        _period = 1;
        period = 1;
    }else if(_period > 65535){
        _period = 65535;
        period = 65535;
    }else{
        period = _period;
    }
    TIM_SetAutoreload(TIM3, period);
}

bool TimerField_accTimerIsRunning(const TimerField* timerfield){
    return timerfield->accTimerRunning;
}

void FUN_IN_RAM TimerField_setPulseWidth(TimerField* timerfield, uint32_t delay){  // us
    // TODO

#if(ALLOCAT_PULSE_TIMER)

    uint32_t timer_clk = 0, period = 0;
    if(delay == 0){
        TimerField_pulseTimerStop(timerfield);
        return;
    }
    timer_clk = STEP_TIM_CLK;
    period = (timer_clk / 1000000) * delay;

    TIM_SetAutoreload(TIM2, period);
#else
    uint16_t ccr = 0;
    if(delay < 2){
        delay = 2;
        ccr = (STEP_TIM_CLK / 1000000) * delay;
    }else if(delay > 7246){
        delay = 7246;
        ccr = (STEP_TIM_CLK / 1000000) * delay;
    }else{
        ccr = (STEP_TIM_CLK / 1000000) * delay;
    }
    TIM_SetCompare1(TIM4, ccr);
#endif

}

void FUN_IN_RAM TimerField_triggerDelay(TimerField* timerfield){
    // TODO
#if(ALLOCAT_PULSE_TIMER)
    
    TIMERFIELD_START(TIM2);

#else
    (void)timerfield;
    TIM_ClearFlag(TIM4, TIM_FLAG_CC1);
    TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);
#endif
    timerfield->pulseTimerRunning = 1;
}

void FUN_IN_RAM TimerField_pulseTimerStop(TimerField* timerfield){
#if(ALLOCAT_PULSE_TIMER)
    
    TIMERFIELD_STOP(TIM2);

#else
     (void)timerfield;
     TIM_ITConfig(TIM4, TIM_IT_CC1, DISABLE);
     TIM_ClearFlag(TIM4, TIM_FLAG_CC1);
#endif
    timerfield->pulseTimerRunning = 0;
}

bool TimerField_pulseTimerIsRunning(const TimerField* timerfield)
{
#if(ALLOCAT_PULSE_TIMER)

    return timerfield->pulseTimerRunning;
#else

    return timerfield->pulseTimerRunning;
#endif

    
}

void FUN_IN_RAM TimerField_timerEndAfterPulse(TimerField *_timerfield){
    // TODO
    _timerfield->lastPulse = 1;
}


