/*
 * @Author: xiaosenluo xiaosenluo@yandex.com
 * @Date: 2023-02-06 00:32:59
 * @LastEditors: xiaosenluo xiaosenluo@yandex.com
 * @LastEditTime: 2023-02-12 20:01:54
 * @FilePath: \wch_ch32v203_TeensyStepC\User\Test\test_tim.c
 * @Description: 
 * 
 * Copyright (c) 2023 by ${git_name_email}, All Rights Reserved. 
 */
/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/


#include "debug.h"
#include "IQmath_RV32.h"
#include "string.h"
#include "ch32v20x_it.h"

/* Global define */

/* Global Variable */
/*********************************************************************
 * @fn      NMI_Handler
 *
 * @brief   This function handles NMI exception.
 *
 * @return  none
 */
void NMI_Handler(void)
{
}

/*********************************************************************
 * @fn      HardFault_Handler
 *
 * @brief   This function handles Hard Fault exception.
 *
 * @return  none
 */
void HardFault_Handler(void)
{
  while (1)
  {
  }
}


/*********************************************************************
 * @fn      GPIO_Toggle_INIT
 *
 * @brief   Initializes GPIOA.0
 *
 * @return  none
 */
void GPIO_Toggle_INIT(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_14 | GPIO_Pin_12;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_5 | GPIO_Pin_3 | GPIO_Pin_1 | GPIO_Pin_10 | GPIO_Pin_12;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void GPIO_Toggle(GPIO_TypeDef *GPIOx, uint16_t Pinx){
    uint32_t state = GPIOx->OUTDR;

    if(state & Pinx){   /// high
        GPIOx->BCR = Pinx;
    }else{
        GPIOx->BSHR = Pinx;
    }
}

void TIM2_IRQHandler(void){
    if(TIM_GetFlagStatus(TIM2, TIM_FLAG_Update) == SET){
        TIM_ClearFlag(TIM2, TIM_FLAG_Update);
        GPIO_Toggle(GPIOB, GPIO_Pin_1);
    }
}
void TIM3_IRQHandler(void){
    if(TIM_GetFlagStatus(TIM3, TIM_FLAG_Update) == SET){
        TIM_ClearFlag(TIM3, TIM_FLAG_Update);
        GPIO_Toggle(GPIOB, GPIO_Pin_10);
    }
}

void TIM4_IRQHandler(void){
    if(TIM_GetFlagStatus(TIM4, TIM_FLAG_Update) == SET){
        TIM_ClearFlag(TIM4, TIM_FLAG_Update);
        GPIO_Toggle(GPIOB, GPIO_Pin_3);
    }
    if(TIM_GetFlagStatus(TIM4, TIM_FLAG_CC1) == SET){
        TIM_ClearFlag(TIM4, TIM_FLAG_CC1);
        GPIO_Toggle(GPIOB, GPIO_Pin_12);
    }
}

#define TIM_CLK_PSC      36000000
#define STEP_TIM_CLK     9000000
#define ACC_TIM_CLK      1000000

#define TIMERFIELD_START(tim)  do{ \
    TIM_ARRPreloadConfig(tim, ENABLE); \
    TIM_ClearFlag(tim, TIM_FLAG_Update); \
    TIM_ITConfig(tim, TIM_IT_Update, ENABLE); \
    TIM_Cmd(tim, ENABLE); \
}while(0)

#define TIMERFIELD_STOP(tim)  do{ \
    TIM_Cmd(tim, DISABLE); \
    TIM_ClearFlag(tim, TIM_FLAG_Update); \
    TIM_ITConfig(tim, TIM_IT_Update, DISABLE); \
}while(0)


/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    SystemCoreClockUpdate();
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);
    Delay_Init();
    USART_Printf_Init(115200);
    GPIO_Toggle_INIT();
    printf("SystemClk:%ld\r\n", SystemCoreClock);


    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure = { 0 };
    TIM_OCInitTypeDef TIM_OCInitStructure={ 0 };
    RCC_ClocksTypeDef RCC_Clocks = { 0 };
    
    uint32_t tim_clk = 144000000, priod = 144, presc = 1;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
    
    RCC_GetClocksFreq(&RCC_Clocks);
    tim_clk = RCC_Clocks.PCLK1_Frequency;
    if(!(RCC->CFGR0 & RCC_PPRE1_DIV1)){
        tim_clk <<= 1;
    }

    tim_clk /= 4;
    presc = (tim_clk / STEP_TIM_CLK) - 1;
    priod = 27;   /// 默认间隔 3US

    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV4;
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInitStructure.TIM_Period = 1000;
    TIM_TimeBaseInitStructure.TIM_Prescaler = 16;

    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseInitStructure);
    TIM_ARRPreloadConfig(TIM2, ENABLE);
    TIM_Cmd(TIM2, DISABLE );
    TIM_ClearFlag(TIM2, TIM_FLAG_Update);


    /// 计数频率 9MHz, 用于输出 Step 脉冲
    TIM_TimeBaseInitStructure.TIM_Period = 1800;
    TIM_TimeBaseInit(TIM4, &TIM_TimeBaseInitStructure);
    TIM_ARRPreloadConfig(TIM4, ENABLE);
    TIM_Cmd(TIM4, DISABLE );
    TIM_ClearFlag(TIM4, TIM_FLAG_Update);


    /// 用于设置加速间隔
    // TIM_TimeBaseInitStructure.TIM_Prescaler = 16;
    TIM_TimeBaseInitStructure.TIM_Period = 800;
    TIM_TimeBaseInit(TIM3, &TIM_TimeBaseInitStructure);
    TIM_ARRPreloadConfig(TIM3, ENABLE);
    TIM_Cmd(TIM3, DISABLE );
    TIM_ClearFlag(TIM3, TIM_FLAG_Update);

    // TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
    // TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
    // TIM_OCInitStructure.TIM_Pulse = 100;   /// 默认宽度 2US
    // TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;

    // TIM_OC1Init( TIM4, &TIM_OCInitStructure );
    // TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Disable);
    // TIM_ClearFlag(TIM4, TIM_FLAG_CC1);
    // TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);
    
    NVIC_SetPriority(TIM2_IRQn, 2);
    NVIC_SetPriority(TIM3_IRQn, 2);
    NVIC_SetPriority(TIM4_IRQn, 2);
    
    NVIC_EnableIRQ(TIM2_IRQn);
    NVIC_EnableIRQ(TIM3_IRQn);
    NVIC_EnableIRQ(TIM4_IRQn);
    
    TIMERFIELD_START(TIM2);
    // TIMERFIELD_START(TIM3);
    TIMERFIELD_START(TIM4);


    while(1)
    {
        Delay_Ms(250);
        GPIO_Toggle(GPIOA, GPIO_Pin_0);
    }
}
