/*
 * @Author: xiaosenluo xiaosenluo@yandex.com
 * @Date: 2023-02-06 00:32:59
 * @LastEditors: xiaosenluo xiaosenluo@yandex.com
 * @LastEditTime: 2023-02-12 17:16:32
 * @FilePath: \wch_ch32v203_TeensyStepC\User\main.c
 * @Description: 
 * 
 * Copyright (c) 2023 by ${git_name_email}, All Rights Reserved. 
 */
/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/


#include "debug.h"
#include "IQmath_RV32.h"
#include "string.h"
/* Global define */

/* Global Variable */

/*********************************************************************
 * @fn      GPIO_Toggle_INIT
 *
 * @brief   Initializes GPIOA.0
 *
 * @return  none
 */
void GPIO_Toggle_INIT(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_14 | GPIO_Pin_12;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_5 | GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void GPIO_Toggle(GPIO_TypeDef *GPIOx, uint16_t Pinx){
    uint32_t state = GPIOx->OUTDR;

    if(state & Pinx){   /// high
        GPIOx->BCR = Pinx;
    }else{
        GPIOx->BSHR = Pinx;
    }
}

#include "bget.h"
static volatile uint8_t __attribute__((aligned(4))) memory_pool[4096] = { 0 };

#define RPM_TO_PPS(rpm, ppr)         ((int32_t)(_IQint(_IQdiv(_IQ((rpm) * (ppr)), _IQ(60)))))

#include "TeensyStepC.h"

StepControl stepControl = { .controller.accUpdatePeriod = 5000, .controller.pulseWidth = 20 };
RotateControl rotateControl = { .controller.accUpdatePeriod = 5000, .controller.pulseWidth = 20 };

Stepper stepper[8] = { 0 };

static void reachedCallback(int32_t pos){
    printf("position: %ld\n", pos);
}

static void errorCallback(Stepper *p, int err){
    printf("error: %d\n", err);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    SystemCoreClockUpdate();
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);
    Delay_Init();
    USART_Printf_Init(115200);
    GPIO_Toggle_INIT();
    printf("SystemClk:%ld\r\n", SystemCoreClock);

    // printf("GPIO Toggle TEST\r\n");
#if(ALLOCAT_PULSE_TIMER)
    TimerField_init(&stepControl.controller.timerField, &(const TimerField_InitTypeDef){
        .stepTimer = TIM4, .accTimer = TIM3, .pulseTimer = TIM2
    });
    NVIC_SetPriority(TIM2_IRQn, 3);
    NVIC_EnableIRQ(TIM2_IRQn);
#else
    TimerField_init(&stepControl.controller.timerField, &(const TimerField_InitTypeDef){
        .stepTimer = TIM4, .accTimer = TIM3, .pulseTimer = TIM4
    });
#endif
    NVIC_SetPriority(TIM3_IRQn, 2);
    NVIC_SetPriority(TIM4_IRQn, 2);

    NVIC_EnableIRQ(TIM3_IRQn);
    NVIC_EnableIRQ(TIM4_IRQn);

    StepControl_init(&stepControl, &(const StepControl_Init_TypeDef){
        .accUpdatePeriod = 5000, .pulseWidth = 5, 
        .reachedTargetCallback = &reachedCallback, .errorCallback = &errorCallback
    });
    
    // RotateControl_init(&rotateControl, &(const StepControl_Init_TypeDef){
    //     .accUpdatePeriod = 5000, .pulseWidth = 10, .reachedTargetCallback = &reachedCallback, .errorCallback = NULL
    // });

    Stepper_init(&stepper[0], &(const Stepper_InitTypeDef){
        .dirPin = {.port = (uint32_t)GPIOB, .pin = GPIO_Pin_7}, 
        .stepPin = {.port = (uint32_t)GPIOB, .pin = GPIO_Pin_5}
    });

    #define PPR     (200 * 32)

    Stepper_setAcceleration(&stepper[0], RPM_TO_PPS(200, PPR));
    Stepper_setMaxSpeed(&stepper[0], RPM_TO_PPS(600, PPR));
    Stepper_setPullInOutSpeed(&stepper[0], \
                                       RPM_TO_PPS(300, PPR), \
                                       RPM_TO_PPS(300, PPR));
    // Stepper_setStepPinPolarity(&stepper[0], 1);
    // Stepper_setInverseRotation(&stepper[0], false);
    Stepper_setTargetAbs(&stepper[0], 10000);
    
    Stepper *ptr = &stepper[0];

    StepControl_move(&stepControl, 1.0, 1, &ptr);
    // RotateControl_rotateAsync(&rotateControl, 1.0f, 1, &ptr);

//     for(int32_t i = 0; i < 32; i++){
// #define NUM        ((1 << i) - i)
//         _iq iqn = _IQsqrt(_IQ(NUM));     
//         float iqf = _IQtoF(iqn);
//         float mf = sqrtf(NUM);
//         printf("_IQsqrt(%ld)=%f, _IQint = %ld, sqrtf(%ld)=%f\n", NUM, iqf, _IQint(iqn), NUM, mf);
// #undef NUM
//     }

//     for(int32_t i = 0; i < 10000; i++){
// #define NUM        (i)
//         _iq iqn = _IQsqrt(_IQ(NUM));     
//         float iqf = _IQtoF(iqn);
//         float mf = sqrtf(NUM);
//         printf("_IQsqrt(%ld)=%f, _IQint = %ld, sqrtf(%ld)=%f\n", NUM, iqf, _IQint(iqn), NUM, mf);
// #undef NUM
//     }
    
    // printf("_IQ(%ld) * -1 = %d\n", 104, _IQint(_IQ(104) * -1));

    while(1)
    {
        Delay_Ms(250);
        GPIO_TypeDef *gpioa = GPIOA;
        GPIO_Toggle(GPIOA, GPIO_Pin_0);
    }
}
